class Typical < Formula
  desc "Data interchange with algebraic data types"
  homepage "https://github.com/stepchowfun/typical"
  url "https://github.com/stepchowfun/typical/archive/v0.9.7.tar.gz"
  sha256 "5a467c3f84dbaaf223b6ab21ad61034583cd48615c89a4cca62dddbc4ca95b91"
  license "MIT"

  depends_on "rust" => :build

  def install
    system "cargo", "install", *std_cargo_args
  end

  test do
    (testpath/"types.t").write <<~EOS
      struct SendEmailRequest {
          to: String = 0
          subject: String = 1
          body: String = 2
      }

      choice SendEmailResponse {
          success = 0
          error: String = 1
      }
    EOS

    assert_empty shell_output("#{bin}/typical generate types.t --rust types.rs --typescript types.ts")

    generated_rust_code = (testpath/"types.rs").read
    generated_typescript_code = (testpath/"types.ts").read

    assert_match "// This file was automatically generated by Typical", generated_rust_code
    assert_match "pub struct SendEmailRequestIn", generated_rust_code
    assert_match "pub struct SendEmailRequestOut", generated_rust_code
    assert_match "pub enum SendEmailResponseIn", generated_rust_code
    assert_match "pub enum SendEmailResponseOut", generated_rust_code
    assert_match "// This file was automatically generated by Typical", generated_typescript_code
    assert_match "export type SendEmailRequestIn", generated_typescript_code
    assert_match "export type SendEmailRequestOut", generated_typescript_code
    assert_match "export type SendEmailResponseIn", generated_typescript_code
    assert_match "export type SendEmailResponseOut", generated_typescript_code
  end
end
